"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  var buildScope = function buildScope(path) {
    var scope = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    var functionParent = path.getFunctionParent();
    if (!functionParent) {
      return scope;
    }
    if (functionParent.isFunctionDeclaration()) {
      scope.push(functionParent.node.id.name);
    } else if (t.isClassProperty(functionParent) || t.isClassMethod(functionParent)) {
      scope.push(functionParent.node.key.name);
      var classBody = functionParent.findParent(function (path) {
        return path.isClassDeclaration() || path.isClassExpression();
      });
      if (classBody) {
        scope.push(classBody.node.id.name);
        return buildScope(classBody, scope);
      }
    } else if (functionParent.isArrowFunctionExpression()) {
      var arrFuncParent = functionParent.findParent(function (path) {
        return path;
      });
      if (t.isVariableDeclarator(arrFuncParent)) {
        scope.push(arrFuncParent.node.id.name);
      } else if (t.isClassProperty(arrFuncParent) || t.isClassMethod(arrFuncParent)) {
        scope.push(arrFuncParent.node.key.name);
        var _classBody = arrFuncParent.findParent(function (path) {
          return path.isClassDeclaration() || path.isClassExpression();
        });
        if (_classBody) {
          scope.push(_classBody.node.id.name);
          return buildScope(_classBody, scope);
        }
      } else if (t.isCallExpression(arrFuncParent)) {
        var callee = arrFuncParent.node.callee;

        scope.push("[" + callee.object.name + "." + callee.property.name + "]");
      }
    }
    return buildScope(functionParent, scope);
  };

  return {
    name: "babel-plugin-captains-log", // not required
    visitor: {
      MemberExpression: function MemberExpression(path) {
        if (path.get("object").isIdentifier({ name: "console" })) {
          var scope = buildScope(path);
          path.parent.arguments.unshift(t.stringLiteral(scope.reverse().join(".") + ":"));
        }
      }
    }
  };
};